# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'EmailTemplate'
        db.create_table(u'classy_mail_emailtemplate', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=64)),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=512)),
            ('subject_content', self.gf('django.db.models.fields.CharField')(max_length=512)),
            ('text_content', self.gf('django.db.models.fields.TextField')()),
            ('html_content', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
        ))
        db.send_create_signal(u'classy_mail', ['EmailTemplate'])

        # Adding model 'CampaignAddressee'
        db.create_table(u'classy_mail_campaignaddressee', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'], null=True, blank=True)),
            ('email_address', self.gf('django.db.models.fields.EmailField')(unique=True, max_length=75)),
            ('first_name', self.gf('django.db.models.fields.CharField')(max_length=128, null=True, blank=True)),
            ('last_name', self.gf('django.db.models.fields.CharField')(max_length=128, null=True, blank=True)),
        ))
        db.send_create_signal(u'classy_mail', ['CampaignAddressee'])

        # Adding model 'EmailCampaign'
        db.create_table(u'classy_mail_emailcampaign', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=64)),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=512)),
            ('template_name', self.gf('django.db.models.fields.CharField')(max_length=128, null=True, blank=True)),
            ('subject_content', self.gf('django.db.models.fields.CharField')(max_length=512, null=True, blank=True)),
            ('text_content', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('html_content', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
        ))
        db.send_create_signal(u'classy_mail', ['EmailCampaign'])

        # Adding M2M table for field emails on 'EmailCampaign'
        m2m_table_name = db.shorten_name(u'classy_mail_emailcampaign_emails')
        db.create_table(m2m_table_name, (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('emailcampaign', models.ForeignKey(orm[u'classy_mail.emailcampaign'], null=False)),
            ('campaignaddressee', models.ForeignKey(orm[u'classy_mail.campaignaddressee'], null=False))
        ))
        db.create_unique(m2m_table_name, ['emailcampaign_id', 'campaignaddressee_id'])

        # Adding model 'CampaignMail'
        db.create_table(u'classy_mail_campaignmail', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('campaign', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['classy_mail.EmailCampaign'])),
            ('adressee', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['classy_mail.CampaignAddressee'])),
            ('date_sent', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('date_opened', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('date_clicked', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
        ))
        db.send_create_signal(u'classy_mail', ['CampaignMail'])


    def backwards(self, orm):
        # Deleting model 'EmailTemplate'
        db.delete_table(u'classy_mail_emailtemplate')

        # Deleting model 'CampaignAddressee'
        db.delete_table(u'classy_mail_campaignaddressee')

        # Deleting model 'EmailCampaign'
        db.delete_table(u'classy_mail_emailcampaign')

        # Removing M2M table for field emails on 'EmailCampaign'
        db.delete_table(db.shorten_name(u'classy_mail_emailcampaign_emails'))

        # Deleting model 'CampaignMail'
        db.delete_table(u'classy_mail_campaignmail')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'classy_mail.campaignaddressee': {
            'Meta': {'object_name': 'CampaignAddressee'},
            'email_address': ('django.db.models.fields.EmailField', [], {'unique': 'True', 'max_length': '75'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        u'classy_mail.campaignmail': {
            'Meta': {'object_name': 'CampaignMail'},
            'adressee': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['classy_mail.CampaignAddressee']"}),
            'campaign': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['classy_mail.EmailCampaign']"}),
            'date_clicked': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'date_opened': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'date_sent': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        u'classy_mail.emailcampaign': {
            'Meta': {'object_name': 'EmailCampaign'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '512'}),
            'emails': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['classy_mail.CampaignAddressee']", 'symmetrical': 'False'}),
            'html_content': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '64'}),
            'subject_content': ('django.db.models.fields.CharField', [], {'max_length': '512', 'null': 'True', 'blank': 'True'}),
            'template_name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'text_content': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'})
        },
        u'classy_mail.emailtemplate': {
            'Meta': {'object_name': 'EmailTemplate'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '512'}),
            'html_content': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '64'}),
            'subject_content': ('django.db.models.fields.CharField', [], {'max_length': '512'}),
            'text_content': ('django.db.models.fields.TextField', [], {})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['classy_mail']


import logging
logger = logging.getLogger("classy_mail")
from mixins.filesystem import FileTemplateMixin
from mixins.model import ModelTemplateMixin
from mixins.log import LogMessageMixin, UniqueMessageMixin
from mixins.md_template import MarkdownTemplateMixin
from .base import BaseEmail


class ModelTemplateEmail(LogMessageMixin, ModelTemplateMixin, BaseEmail):
    pass


class FileTemplateEmail(LogMessageMixin, FileTemplateMixin, BaseEmail):
    pass


class MarkdownTemplateEmail(LogMessageMixin, MarkdownTemplateMixin, BaseEmail):
    pass

class UniqueMarkdownTemplateEmail(UniqueMessageMixin, MarkdownTemplateMixin, BaseEmail):
    pass



import logging
logger = logging.getLogger("classy_mail")
from django.conf import settings
from django.db import models

class EmailTemplate(models.Model):
    name = models.CharField(max_length=64, unique=True)
    description = models.CharField(max_length=512)
    subject_content = models.CharField(max_length=512)
    text_content = models.TextField()
    html_content = models.TextField(null=True, blank=True)

    def __unicode__(self):
        return self.name


class CampaignAddressee(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)
    email_address = models.EmailField(unique=True)
    first_name = models.CharField(max_length=128, null=True, blank=True)
    last_name = models.CharField(max_length=128, null=True, blank=True)


class EmailCampaign(models.Model):
    name = models.CharField(max_length=64, unique=True)
    description = models.CharField(max_length=512)
    template_name = models.CharField(max_length=128, null=True, blank=True)
    subject_content = models.CharField(max_length=512, blank=True, null=True)
    text_content = models.TextField(blank=True, null=True)
    html_content = models.TextField(null=True, blank=True)
    emails = models.ManyToManyField(CampaignAddressee)


class CampaignMail(models.Model):
    campaign = models.ForeignKey(EmailCampaign)
    adressee = models.ForeignKey(CampaignAddressee)
    date_sent = models.DateTimeField(null=True, blank=True)
    date_opened = models.DateTimeField(null=True, blank=True)
    date_clicked = models.DateTimeField(null=True, blank=True)

# TODO: Add the tracking pixel and a link interceptor

class EmailMessageLog(models.Model):
    message_text = models.TextField(blank=True, null=True)
    subject = models.CharField(max_length=512, blank=True, null=True)
    from_header = models.CharField(max_length=512, blank=True, null=True)
    to_header = models.CharField(max_length=512, blank=True, null=True)
    cc_header = models.CharField(max_length=512, blank=True, null=True)
    bcc_header = models.CharField(max_length=512, blank=True, null=True)
    date_header = models.CharField(max_length=512, blank=True, null=True)
    sent = models.DateTimeField(auto_now_add=True)



from django.conf import settings


# Exceptions
class ImpossibleEmail(Exception):
    pass


class NoFromAddress(ImpossibleEmail):
    pass


class NoToAddress(ImpossibleEmail):
    pass


class NoSubject(ImpossibleEmail):
    pass


class NoBody(ImpossibleEmail):
    pass


# Utility Functions
def _makelist(arg):
    if isinstance(arg, list):
        return arg
    if isinstance(arg, basestring):
        return [arg]
    raise TypeError('%s is not a string or a list' % unicode(arg))


# The good stuff

class BaseEmail(object):
    _msg = None
    from_email = settings.DEFAULT_FROM_EMAIL
    context = dict()
    #TODO: Separate getting the template from rendering it

    def __init__(self, **kwargs):
        """
        Constructor. Can contain helpful extra
        keyword arguments, and other things.
        """
        # Go through keyword arguments, and either save their values to our
        # instance, or raise an error.
        for key, value in kwargs.iteritems():
            setattr(self, key, value)

    def get_subject(self):
        try:
            return self.subject
        except AttributeError:
            raise NoSubject("Missing a subject.  try adding it as a kwarg")

    def get_to(self):
        try:
            addr = self.to_addr
        except AttributeError:
            raise NoToAddress("Missing a to_addr.  try adding it as a kwarg")
        return _makelist(addr)

    def get_from_email(self):
        try:
            return self.from_email
        except AttributeError:
            raise NoFromAddress("Missing a from_email.  try adding it as a kwarg")

    def get_cc(self):
        return None

    def get_bcc(self):
        return None

    def get_headers(self):
        return None

    def get_text_content(self):
        try:
            return self.text_content
        except AttributeError:
            raise NoBody("Missing text_content.  try adding it as a kwarg")

    def get_html_content(self):
        return None

    def send(self):
        self.msg = self.build_msg()
        self.msg.send()


class SimpleEmail(BaseEmail):
    """This email needs everything passed in to init"""

    def build_msg(self):
        if self._msg is None:
            self._msg = EmailMultiAlternatives(
                subject = self.get_subject(),
                body= self.get_text_content(),
                from_email = self.get_from_email(),
                to = self.get_to(),
                bcc = self.get_bcc(),
                cc = self.get_cc(),
                headers = self.get_headers(),
            )
            if self.get_html_content() is not None:
                self._msg.attach_alternative(self.get_html_content(), 'text/html')
        return self._msg

